<?php
/**
 * @author Ahmad Hejazee <mngafa@gmail.com>
 */

/**
 * Simple api provides a simple and nearly backward compatible API
 * for processing ngslaccess files
 */
class NGSLAccessByFileAPISimple extends NGSLAccessByFileAPI {

  /**
   * Return API name
   */
  public function getAPIName() {
    return 'simple';
  }

  /**
   * Required string 'type' and array 'arguments[]' directives
   * be present in [ngslaccess] section
   */
  protected function requiredDirectives() {
    return array(
      'handler'   => array('type' => 'string'),
      'arguments' => array('type' => 'array'),
    );
  }

  /**
   * Does not require any sections
   */
  protected function requiredSections() {
    return array();
  }

  /**
   * Validate the schema
   */
  protected function validateSchema() {
    // The simple API does not need any specific schema.
    // All requirements are already checked.
    // So we don't have anything to do here.
    // Not throwing exceptions, means a valid schema.
  }

  /**
   * @return bool
   * @throws \NGSLAccessByFileAPI_HanlerClassNotFound
   * @throws \NGSLAccessByFileAPI_SchemaReadValueException
   * @throws \NGSLAccessByFileHandler_MissingArgument
   */
  public function hasAccess() {
    $handler_name = $this->get('handler');
    $arguments = $this->get('arguments');

    // Determine and get the handler class
    $handler_class = ngsl_access_byfile_handlers_get($handler_name);

    if (!$handler_class) {
      throw new NGSLAccessByFileAPI_HanlerClassNotFound($handler_name);
    }

    // Create the handler object and run the access checker.
    $handler = new $handler_class();
    $result = $this->runHandler($handler, $arguments);

    return $result;
  }

  /**
   * Run the access handler and return the boolean result.
   *
   * @param \NGSLAccessByFileHandler $handler
   * @param array $args
   *
   * @return bool Whether the access is granted or not
   * @throws \NGSLAccessByFileHandler_MissingArgument
   */
  private function runHandler(NGSLAccessByFileHandler $handler, array $args) {
    $handler->addArguments($args);
    $result = $handler->run();

    return $result;
  }
}
